
/**
 * <h1>Clase principal del programa.</h1> 
 * Implementa los constructores y métodos esenciales que permiten el inicio del
 * programa, así como la interacción entre todas las clases. 
 * <b>Nota:<b> Se implementa el Stage4 junto a la parte extra. Ambas partes se hacen efectivas una vez cerrado el programa, es decir,
 * una vez cerrada la aplicación se crean los archivos tipo PBM con las rutas de los robots y los archivos csv con los respectivos datos.
 * 
 * @author Matías Contreras, Francisco Frez, Damián Quiroz.
 * @version 2.0
 * @since 2018-04-30
*/
import java.awt.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

public class Stage4 {
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() { // implementaci�n Swing recomendada
			public void run() {
				MainFrame frame = new MainFrame();
				frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				frame.setVisible(true);
				frame.setResizable(true);
				frame.addWindowListener(frame);
			} // run ends
		});
	}
}

/**
 * A frame containing the application main GUI
 */
class MainFrame extends JFrame implements WindowListener {
	public MainFrame() {
		this.setTitle("ELO329: Robots en Laberinto");
		this.setSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);

		/* Obtiene dimensiones de pantalla */
		Toolkit myScreen = Toolkit.getDefaultToolkit();
		Dimension screenSize = myScreen.getScreenSize();

		int widthScreen = screenSize.width;
		int heigthScreen = screenSize.height;

		/* Ventana inicia en el centro */
		this.setLocation(widthScreen / 2 - DEFAULT_WIDTH / 2, heigthScreen / 2 - DEFAULT_HEIGHT / 2);

		/* Icono de la ventana */
		Image negroIcono = myScreen.getImage("images/icono_negro.png");
		this.setIconImage(negroIcono);

		panel = new JPanel();
		this.add(panel);
		panel.setLayout(new BorderLayout());

		/* Se crea el boton */
		button = new MyTime();
		play = new JButton();
		play = button.getView();
		/* Color del boton */
		play.setBackground(Color.WHITE);

		/* Se agrega el boton */
		panel.add(play, BorderLayout.SOUTH);
		this.validate();
		this.menuBar = new MainMenuBar(panel, this);
		/* Agregar barra de menu */
		setJMenuBar(menuBar);
	}

	public MyTime getButton() {
		return this.button;
	}

	/*
	 * Se agregan los métodos de windowListener para guardar los archivos cuando se
	 * cierre la aplicación.
	 */
	@Override
	public void windowOpened(WindowEvent e) {
		System.err.println("Me abriste we xdXD");

	}

	@Override
	public void windowClosing(WindowEvent e) {
		this.menuBar.getMainPanel().write();

	}

	@Override
	public void windowClosed(WindowEvent e) {
		// this.menuBar.getMainPanel().write();
	}

	@Override
	public void windowIconified(WindowEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void windowDeiconified(WindowEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void windowActivated(WindowEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void windowDeactivated(WindowEvent e) {
		// TODO Auto-generated method stub

	}

	private static final int DEFAULT_WIDTH = 600;
	private static final int DEFAULT_HEIGHT = 600;
	private JPanel panel;
	private MyTime button;
	private JButton play;
	private MainMenuBar menuBar;
}

/**
 * Clase que contiene la barra de herramientas de la aplicación.
 */

class MainMenuBar extends JMenuBar implements ActionListener {
	/**
	 * Constructor de la clase. Se usa para pasar algunos parámetros de interés.
	 * 
	 * @param panel
	 *            : Variable del tipo JPanel, entrega a la clase el panel principal
	 *            de la ventana inicial.
	 * @param mainFrame
	 *            : Variable del tipo MainFrame, entrega a la clase el Frame
	 *            principal de la ventana.
	 */
	public MainMenuBar(JPanel panel, MainFrame mainFrame) {
		this.panel = panel;
		this.mainFrame = mainFrame; // Deberia poder usarse para ajustar la ventana cuando ingresa el maze
		this.mainPanel = new MyWorld();
		JMenu menu_file = new JMenu("File");
		JMenu menu_world = new JMenu("World");
		JMenuItem item = new JMenuItem("Open");
		JMenuItem item_create_robot = new JMenuItem("Create Robot");
		JMenuItem item_delta = new JMenuItem("Set delta t");
		menu_file.add(item);
		menu_world.add(item_create_robot);
		menu_world.add(item_delta);
		add(menu_file);
		add(menu_world);

		/* Barra de menu como listener de item open */
		item.addActionListener(this);

		robotConfigurationListener = new RobotConfigurationListener();
		item_create_robot.addActionListener(robotConfigurationListener);

		deltaTListener = new DeltaTListener(mainPanel);
		item_delta.addActionListener(deltaTListener);
	}

	public MyWorld getMainPanel() {
		return this.mainPanel;
	}

	public void actionPerformed(ActionEvent e) {
		fc = new JFileChooser();
		fc.setFileFilter(new FileNameExtensionFilter("PBM file", "pbm"));
		int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File file = fc.getSelectedFile();
			try {
				/* Se lee el laberinto */
				inMaze = new Scanner(file);
				maze = new Maze(inMaze);

				/* Se crea la lamina donde va el laberinto */
				// mainPanel = new MyWorld();
				mainPanel.setMaze(maze);
				mainPanel.setButton(mainFrame.getButton());
				mainPanel.setLayout(new BorderLayout());

				/* Se agrega la lamina al frame principal */
				panel.add(mainPanel, BorderLayout.CENTER);
				panel.validate();

				/* Parametros para setear el tamanho de la ventana. */
				int scale = mainPanel.getScale();
				int initxy = 0;
				int dimx = scale * maze.getWidth();
				int dimy = scale * maze.getHeight();
				mainFrame.setSize(new Dimension(2 * initxy + dimx, initxy + dimy + scale * 128));

			} catch (IOException ex) {
				System.out.println("No se encontro el archivo");
			}
		} else {
			System.out.println("Open command cancelled by user.");
		}

	}

	/** Clase interna que permite configurar las variables de los robots creados. */
	private class RobotConfigurationListener implements ActionListener {
		/**
		 * Constructor base de la clase, solamente sirve para crear un objeto del tipo
		 * RobotConfigurationListener
		 */
		public RobotConfigurationListener() {
		}

		public void actionPerformed(ActionEvent e) {
			// Create and set up the window.
			robotConfigurationFrame = new RobotConfigurationFrame();
			robotConfigurationFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			textField_vx = robotConfigurationFrame.getTextField_vx();
			textField_vy = robotConfigurationFrame.getTextField_vy();
			textField_pilot = robotConfigurationFrame.getTextField_pilot();
			errorLabel = robotConfigurationFrame.getErrorLabel();
			configurationButton = robotConfigurationFrame.getConfigurationButton();

			/* Boton como listener */
			RobotCreationListener ButtonListener = new RobotCreationListener(robotConfigurationFrame);
			configurationButton.addActionListener(ButtonListener);
		}
	}

	/**
	 * Clase interna que permite actuar ante eventos de la clase
	 * RobotConfiguracionListener.
	 */
	private class RobotCreationListener implements ActionListener {

		public RobotCreationListener(RobotConfigurationFrame robotConfigurationFrame) {
			this.robotConfigurationFrame = robotConfigurationFrame;
			robotList = new ArrayList<Robot>();
			rPanel = new RobotView();

		}

		public void actionPerformed(ActionEvent e) {
			/*
			 * Principales modificaciones son: Agregar las excepciones necesarias para cada
			 * etapa.
			 */
			try {
				vx = Float.parseFloat(textField_vx.getText());
				vy = Float.parseFloat(textField_vy.getText());
				pilot = Float.parseFloat(textField_pilot.getText());

			} catch (java.lang.NumberFormatException ex) {
				errorLabel.setText("Ingrese datos validos!");
				return;
			}

			String stype;
			if (pilot == 1) {
				stype = "A";
			} else if (pilot == 0) {
				stype = "P";
			} else {
				errorLabel.setText("Pilot invalido!");
				return;
			}

			try {

				if (mainFrame.getButton().getState() == true) {
					errorLabel.setText("Detenga la simulacion!");
					return;
				} else if (vx != 0 && vy != 0) {
					errorLabel.setText("Velocidades invalidas!");
					return;
				} else {
					robot = new Robot();
					robot.getVelocity().setTo(vx, vy);
					robot.setSRange(3);
					robot.setWorld(mainPanel);
					robotList.add(robot);
					rPanel = robot.getGraphicRobot();
					rPanel.setLayout(new BorderLayout());
					rPanel.setOpaque(false);
					MyPilot px = new MyPilot(robot, stype);
					mainPanel.setRobot(robot, px);
					mainPanel.add(rPanel, BorderLayout.CENTER);
					mainPanel.revalidate();

					robotConfigurationFrame.dispose();
					System.out.println("vx = " + vx);
					System.out.println("vy = " + vy);
					System.out.println("pilot = " + pilot);
					System.out.println("Se ha creado un robot.");
				}
			} catch (java.lang.NullPointerException ex) {
				errorLabel.setText("No hay laberinto!");
				return;
			}
		}

		private Robot robot;
		private RobotView rPanel;
		private RobotConfigurationFrame robotConfigurationFrame;

	}

	private class DeltaTListener implements ActionListener {

		public DeltaTListener(MyWorld inWorld) {
			this.world = inWorld;
		}

		public void actionPerformed(ActionEvent e) {
			if (world != null) {
				deltaFrame = new DeltaTFrame(world);
				deltaFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
				deltaFrame.setVisible(true);
			} else {
				System.err.println("No se ha seteado el laberinto");
				return;
			}

		}

		private DeltaTFrame deltaFrame;
		private MyWorld world;
	}

	private ArrayList<Robot> robotList;

	private JFileChooser fc;
	private Scanner inMaze;
	private Maze maze;
	private JPanel panel;
	private MyWorld mainPanel;
	private DeltaTListener deltaTListener;
	private MainFrame mainFrame;
	private RobotConfigurationListener robotConfigurationListener;
	private RobotConfigurationFrame robotConfigurationFrame;
	private JTextField textField_vx;
	private JTextField textField_vy;
	private JTextField textField_pilot;
	private JLabel errorLabel;
	private JButton configurationButton;
	private float vx;
	private float vy;
	private float pilot;

}
