/**
 * <h1>Vector en el plano cartesiano</h1> Implementa un vector con coordenadas
 * (X,Y) en un plano cartesiano, con las operaciones necesarias para realizar
 * distintas tareas.
 * <p>
 * <b>Nota:</b> Sin modificaciones desde la version anterior, entregada en la
 * Tarea 1 del ramo ELO329.
 * 
 * @author Matias Contreras, Francisco Frez, Damin Quiroz.
 * @version 1.0
 * @since 2018-04-13
 */

public class Vector2D {
	private double x, y;
	// we will use cartesian coordinates

	/** Constructor Base de la clase, se inicializan las variables en 0. */
	public Vector2D() {
		x = y = 0;
	}

	/**
	 * Constructoe secundario de la clase, inicializa las variables en un valor
	 * arbitrario (X,Y).
	 * 
	 * @param x:
	 *            Punto X del vector inicializado.
	 * @param y:
	 *            Punto Y del vector inicializado.
	 */
	public Vector2D(double x, double y) {
		this.x = x;
		this.y = y;
	}

	/**
	 * Metodo que retorna la coordenada X de un vector.
	 * 
	 * @return double x : Componente X del vector.
	 */
	public double getX() {
		return x;
	}

	/**
	 * Metodo que retorna la coordenada Y de un vector.
	 * 
	 * @return double y : Componente Y del vector.
	 */
	public double getY() {
		return y;
	}

	/**
	 * Metodo que implementa la normalizacin de un vector. Entrega el vector
	 * normalizado.
	 * 
	 * @return Vector2D norma : Vector normalizado.
	 */
	public Vector2D getNormalize() {
		double module = this.getModule();
		Vector2D out = new Vector2D(this.x / module, this.y / module);
		return out;
		/* normaliza vectores */
	}

	/**
	 * Una segunda opcion para el metodo getNormalize(), entrega un vector
	 * normalizado en una direccion arbitraria.
	 * 
	 * @return Vector2D norma: Vector normalizado.
	 */
	public Vector2D getUnitary() {
		return times(1 / getModule());
	}

	/**
	 * Metodo que permite obtener el mdulo de un vector cualquiera.
	 * 
	 * @return double modulo : Mdulo del vector.
	 */
	public double getModule() {
		return Math.sqrt(x * x + y * y);
	}

	/**
	 * Setea las componentes del vector en un valor arbitrario (X,Y).
	 * 
	 * @param x
	 *            Variable del tipo double, indica la nueva coordenada X del vector.
	 * @param y
	 *            Variable del tipo double, indica la nueva coordenada Y del vector.
	 */
	public void setTo(double x, double y) { // to set a new position
		this.x = x;
		this.y = y;
	}

	/**
	 * Metodo para cambiar el sentido de un vector hacia su izquierda. <b>Nota:</b>
	 * El plano utilizado para calcular el giro, est centrado en el punto (0,0),
	 * con su eje X apuntando hacia la derecha y su eje Y positivo apuntando hacia
	 * abajo.
	 * 
	 * @return Vector2D leftVector : Vector apuntando a su izquierda.
	 */
	public Vector2D turnLeft() {
		return new Vector2D((this.getY()), -this.getX());
		/* giro a la izquierda */
	}

	/**
	 * Metodo para cambiar el sentido de un vector hacia su derecha. <b>Nota:</b> El
	 * plano utilizado para calcular el giro, est centrado en el punto (0,0), con
	 * su eje X apuntando hacia la derecha y su eje Y positivo apuntando hacia
	 * abajo.
	 * 
	 * @return Vector2D rightVector : Vector apuntando a su derecha.
	 */
	public Vector2D turnRight() {
		return new Vector2D(-this.getY(), (this.getX()));
		/* giro a la derecha */
	}

	/*
	 * public void turnLeft() { setTo(-y, x); } public void turnRight() { setTo(y,
	 * -x); }
	 */

	/**
	 * Metodo que implementa la suma cartesiana entre dos vectores.
	 * 
	 * @param v
	 *            : Variable de tipo Vector2D que se sumar al vector original.
	 * @return Vector2D sumVector : Suma cartesiana entre el vector original y el
	 *         vector parmetro de entrada.
	 */
	public Vector2D plus(Vector2D v) {
		if (v == null)
			return new Vector2D(x, y);
		else
			return new Vector2D(x + v.x, y + v.y);
	}

	/**
	 * Metodo que implementa la multiplicacion por un escalar de un vector.
	 * 
	 * @param scalar
	 *            : Variable de tipo double que multipicar al Vector2D original.
	 * @return Vector2D kVector : Vector escalado en "scalar" veces.
	 */
	public Vector2D times(double scalar) {
		return new Vector2D(x * scalar, y * scalar);
	}

	/**
	 * Metodo que implementa la resta cartesiana da dos vectores.
	 * 
	 * @param v
	 *            : Variable del tipo Vector2D que se restar de forma cartesiana al
	 *            vector origianl.
	 * @return Vector2D minVector : Resta cartesiana entre el vector original y el
	 *         vector parmetro de entrada.
	 */
	public Vector2D minus(Vector2D v) {
		if (v == null)
			return new Vector2D(x, y);
		else
			return new Vector2D(x - v.x, y - v.y);
	}

	/**
	 * Metodo que retorna la distancia de un vector, a otro vector.
	 * 
	 * @param p
	 *            : Variable del tipo Vector2D desde la que se calcular la
	 *            distancia cartesiana.
	 * @return double distance : Distancia cartesiana desde el vector original al
	 *         vector parmetro de entrada.
	 */
	public double distanceTo(Vector2D p) {
		double dx = p.x - x;
		double dy = p.y - y;
		return Math.sqrt(dx * dx + dy * dy);
	}

	/**
	 * Metodo descriptivos de la clase. Retorna un string con la informacin X Y.
	 * 
	 * @return String description: Descripcin de las coordenadas.
	 */
	public String getDescription() {
		return "x,\ty";
	}

	/**
	 * Metodo descriptivos de la clase. Retorna un string con la informacin X Y.
	 * 
	 * @return String description: Descripcin de las coordenadas actuales del
	 *         Vector2D.
	 */
	public String toString() {
		return x + ",\t" + y;
	}
}
